@@ -14,15 +14,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ******************************************************************************
+
+__all__ = ["Add", "QuantizedAdd", "QuantizedConcatenate"]
+
 import keras
 import tensorflow as tf
 
-from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH
+from .decorators import (register_quantize_target, rescale_outputs, register_no_output_quantizer,
+                         register_aligned_inputs)
 from .quantizers import OutputQuantizer
 from .recorders import TensorRecorder
-
-
-__all__ = ["Add", "QuantizedAdd", "QuantizedConcatenate"]
+from ..tensors import FixedPoint, BASE_BUFFER_BITWIDTH
 
 
 @tf.keras.utils.register_keras_serializable()
@@ -56,6 +58,8 @@ class Add(keras.layers.Layer):
         return config
 
 
+@register_quantize_target(Add)
+@register_aligned_inputs
 @tf.keras.utils.register_keras_serializable()
 class QuantizedAdd(Add):
     """Sums two inputs and quantize the output.
@@ -65,14 +69,14 @@ class QuantizedAdd(Add):
     The outputs are quantized according to the specified quantization configuration.
 
     Args:
-        quant_config (dict): the serialized quantization configuration.
+        quant_config (dict, optional): the serialized quantization configuration. Defaults to None.
     """
 
-    def __init__(self, *args, quant_config, **kwargs):
+    def __init__(self, *args, quant_config=None, **kwargs):
         super().__init__(*args, **kwargs)
         self.quant_config = quant_config
-        self.buffer_bitwidth = self.quant_config.get(
-            "buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
+        quant_config = quant_config or dict()
+        self.buffer_bitwidth = BASE_BUFFER_BITWIDTH - 1
         out_quant_cfg = quant_config.get("output_quantizer", False)
         if out_quant_cfg:
             self.out_quantizer = OutputQuantizer(
@@ -83,6 +87,7 @@ class QuantizedAdd(Add):
         self.a_shift = TensorRecorder()
         self.b_shift = TensorRecorder()
 
+    @rescale_outputs
     def call(self, inputs):
         a, b = inputs
         if not (isinstance(a, FixedPoint) and isinstance(b, FixedPoint)):
@@ -102,8 +107,6 @@ class QuantizedAdd(Add):
         self.a_shift(shift_ab)
         self.b_shift(shift_ba)
 
-        if self.out_quantizer is not None:
-            outputs = self.out_quantizer(outputs)
         return outputs
 
     def get_config(self):
@@ -112,6 +115,9 @@ class QuantizedAdd(Add):
         return config
 
 
+@register_quantize_target(keras.layers.Concatenate)
+@register_no_output_quantizer
+@register_aligned_inputs
 @tf.keras.utils.register_keras_serializable()
 class QuantizedConcatenate(keras.layers.Concatenate):
     """ A Concatenate layer that operates on quantized inputs
