@@ -14,16 +14,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ******************************************************************************
+
+__all__ = ["QuantizedLayerNormalization", "LayerMadNormalization"]
+
 import tensorflow as tf
 import keras
 from tensorflow.python.framework import ops
 
+from .decorators import (register_quantize_target, rescale_outputs, tensor_inputs,
+                         register_aligned_inputs)
 from .recorders import TensorRecorder
 from .quantizers import AlignedWeightQuantizer, WeightQuantizer, OutputQuantizer
-from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH
-
-
-__all__ = ["QuantizedLayerNormalization", "LayerMadNormalization"]
+from ..tensors import FixedPoint, BASE_BUFFER_BITWIDTH
 
 
 @tf.keras.utils.register_keras_serializable()
@@ -114,15 +116,18 @@ class LayerMadNormalization(keras.layers.LayerNormalization):
         return outputs
 
 
+@register_quantize_target(LayerMadNormalization)
+@register_aligned_inputs
 @tf.keras.utils.register_keras_serializable()
 class QuantizedLayerNormalization(LayerMadNormalization):
     """ A LayerNormalization layer that operates on quantized inputs and weights.
 
     Args:
-        quant_config (dict): the serialized quantization configuration.
+        quant_config (dict, optional): the serialized quantization configuration. Defaults to None.
     """
-    def __init__(self, *args, quant_config, **kwargs):
+    def __init__(self, *args, quant_config=None, **kwargs):
         super().__init__(*args, **kwargs)
+        quant_config = quant_config or dict()
         self.quant_config = quant_config
         out_quant_cfg = quant_config.get("output_quantizer", False)
         if out_quant_cfg:
@@ -130,18 +135,18 @@ class QuantizedLayerNormalization(LayerMadNormalization):
                 name="output_quantizer", **out_quant_cfg)
         else:
             self.out_quantizer = None
-        self.buffer_bitwidth = self.quant_config.get(
-            "buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
-        assert self.buffer_bitwidth > 0, "The buffer_bitwidth must be a strictly positive integer."
-        self.gamma_quantizer = WeightQuantizer(
-            name="gamma_quantizer", **quant_config["gamma_quantizer"])
-        self.beta_quantizer = AlignedWeightQuantizer(
-            name="beta_quantizer", **quant_config["beta_quantizer"])
+        self.buffer_bitwidth = BASE_BUFFER_BITWIDTH - 1
+        gamma_quantizer_cfg = quant_config.get("gamma_quantizer", {"bitwidth": 8})
+        self.quant_config["gamma_quantizer"] = gamma_quantizer_cfg
+        self.gamma_quantizer = WeightQuantizer(name="gamma_quantizer", **gamma_quantizer_cfg)
+        beta_quantizer_cfg = quant_config.get("beta_quantizer", {})
+        self.beta_quantizer = AlignedWeightQuantizer(name="beta_quantizer", **beta_quantizer_cfg)
 
         # Add objects that will store the shift values.
-        self.input_shift = TensorRecorder()
         self.gamma_shift = TensorRecorder()
 
+    @tensor_inputs([FixedPoint])
+    @rescale_outputs
     def call(self, inputs):
         """The quantized version of the LayerNormalization with Integer-only operations.
 
@@ -153,21 +158,6 @@ class QuantizedLayerNormalization(LayerMadNormalization):
         Returns:
             :obj:`FixedPoint`: output tensor.
         """
-        # raise an error if the inputs are not FixedPoint
-        if not isinstance(inputs, FixedPoint):
-            raise TypeError(f"QuantizedLayerNormalization only accepts FixedPoint\
-                              inputs. Receives {type(inputs)} inputs.")
-
-        # Store the initial number of input bits
-        input_bits = inputs.value_bits
-
-        # Expand the inputs to a higher bitwidth to avoid saturation and align them
-        inputs, shift = inputs.expand(self.buffer_bitwidth)
-        self.input_shift(shift)
-
-        # Evaluate the maximum number of bits required to store the expanded inputs
-        input_bits += tf.reduce_max(shift)
-
         # Compute the sum of Absolute Deviations
         mad_sum = tf.math.reduce_sum(
             inputs.abs(), self.axis, keepdims=True, name="mad")
@@ -177,31 +167,24 @@ class QuantizedLayerNormalization(LayerMadNormalization):
         # mad = mad_sum / nb_channels
         # y = x / mad * gamma
         #
-        # We rather directly evaluate the resulting projection scale and multiply:
+        # We rather directly evaluate the resulting projection p and multiply:
         #
-        # scale = gamma * nb_channels / mad_sum
-        # y = x * scale
+        # gamma = gamma * nb_channels
+        # p = gamma / mad_sum
+        # y = x * p
 
-        # The first step is to evaluate the projection scale
+        # The first step is to quantize the rescaled gamma
         nb_channels = tf.cast(inputs.shape[self.axis[0]], tf.float32)
-        gamma = self.gamma * nb_channels
-        # Quantize the rescaled gamma
-        gamma = self.gamma_quantizer(gamma)
+        gamma = self.gamma_quantizer(self.gamma * nb_channels)
 
-        # Evaluate the maximum number of bits allowed for the projection scale to
-        # avoid a saturation of the accumulation buffer, assuming the worst-case
-        # scenario where at least on input uses all input bits.
-        scale_bits = self.buffer_bitwidth - input_bits
+        # We need to upscale gamma before dividing by mad_sum to reduce the error.
+        # For that, we evaluate the maximum number of bits allowed to avoid a saturation.
+        # The final operation corresponds to y = gamma * x / mad_sum
+        # But we know that mad_sum is always > x, so y will never require more bits than gamma.
+        gamma_bits = self.buffer_bitwidth
         if self.out_quantizer is not None and self.out_quantizer.scale_bits is not None:
             # We need to leave some space for the multiplication by the output scales
-            scale_bits -= self.out_quantizer.scale_bits
-
-        # We previously assumed at least one input uses all input_bits, so the minimum
-        # number of bits required to store the mad_sum is input_bits.
-        mad_sum_bits = input_bits
-
-        # Deduce the maximum number of bits allowed for gamma before division by mad_sum
-        gamma_bits = scale_bits + mad_sum_bits
+            gamma_bits -= self.out_quantizer.scale_bits
 
         # Upscale rescaled gamma as much as we can to reduce the division error
         gamma_int_bits = gamma.value_bits - gamma.frac_bits
@@ -209,23 +192,17 @@ class QuantizedLayerNormalization(LayerMadNormalization):
         gamma, shift = gamma.upscale(gamma_frac_bits, self.buffer_bitwidth)
         self.gamma_shift(shift)
 
-        # Calculate the projection scale. This can be evaluated once per token on hardware.
-        scale = tf.math.divide(gamma, mad_sum)
+        # Calculate the projection p. This can be evaluated once per token on hardware.
+        p = tf.math.divide(gamma, mad_sum)
 
         # Apply the projection on inputs
-        inputs_rescaled = tf.math.multiply(inputs, scale)
+        inputs_rescaled = tf.math.multiply(inputs, p)
 
         # Quantize and align beta on the rescaled inputs
         beta = self.beta_quantizer(self.beta, inputs_rescaled)
 
         # Evaluate outputs
-        outputs = tf.add(inputs_rescaled, beta)
-
-        # If no quantizer output is provided return the outputs directly
-        if self.out_quantizer is None:
-            return outputs
-
-        return self.out_quantizer(outputs)
+        return tf.add(inputs_rescaled, beta)
 
     def get_config(self):
         config = super().get_config()
