@@ -14,29 +14,29 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ******************************************************************************
+
+__all__ = ["QuantizedMaxPool2D", "QuantizedGlobalAveragePooling2D"]
+
 import tensorflow as tf
 
 from keras.layers import MaxPool2D, GlobalAveragePooling2D
 from keras.utils import conv_utils
 
+from .decorators import (register_quantize_target, register_no_output_quantizer, rescale_outputs,
+                         tensor_inputs)
 from .quantizers import OutputQuantizer
-from ..tensors import QTensor, MAX_BUFFER_BITWIDTH
-
-__all__ = ["QuantizedMaxPool2D", "QuantizedGlobalAveragePooling2D"]
+from ..tensors import FixedPoint, QTensor, MAX_BUFFER_BITWIDTH
 
 
+@register_quantize_target(MaxPool2D)
+@register_no_output_quantizer
 @tf.keras.utils.register_keras_serializable()
 class QuantizedMaxPool2D(MaxPool2D):
     """A max pooling layer that operates on quantized inputs.
 
     """
-
+    @tensor_inputs([QTensor])
     def call(self, inputs):
-        # Raise an error if the inputs are not QTensor
-        if not isinstance(inputs, QTensor):
-            raise TypeError(f"QuantizedMaxPool2D only accepts QTensor inputs. \
-                             Receives {type(inputs)} inputs.")
-
         if self.data_format == "channels_last":
             ksize = (1,) + self.pool_size + (1,)
             strides = (1,) + self.strides + (1,)
@@ -51,45 +51,41 @@ class QuantizedMaxPool2D(MaxPool2D):
         return outputs
 
 
+@register_quantize_target(GlobalAveragePooling2D)
 @tf.keras.utils.register_keras_serializable()
 class QuantizedGlobalAveragePooling2D(GlobalAveragePooling2D):
     """A global average pooling layer that operates on quantized inputs.
 
      Args:
-        quant_config (dict): the serialized quantization configuration.
+        quant_config (dict, optional): the serialized quantization configuration. Defaults to None.
     """
 
-    def __init__(self, quant_config, **kwargs):
+    def __init__(self, quant_config=None, **kwargs):
         super().__init__(**kwargs)
         self.quant_config = quant_config
+        quant_config = quant_config or dict()
         out_quant_cfg = quant_config.get("output_quantizer", False)
         if out_quant_cfg:
             self.out_quantizer = OutputQuantizer(
                 name="output_quantizer", **out_quant_cfg)
         else:
             self.out_quantizer = None
-        self.buffer_bitwidth = self.quant_config.get("buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
-        assert self.buffer_bitwidth > 0, "The buffer_bitwidth must be a strictly positive integer."
+        self.buffer_bitwidth = MAX_BUFFER_BITWIDTH
 
     def build(self, input_shape):
         super().build(input_shape)
         # Build the spatial size
         self.spatial_size = (input_shape[1] * input_shape[2])
 
+    @tensor_inputs([QTensor])
+    @rescale_outputs
     def call(self, inputs):
-        # Raise an error if the inputs are not QTensor
-        if not isinstance(inputs, QTensor):
-            raise TypeError(f"QuantizedGlobalAveragePooling2D only accepts QTensor inputs. \
-                             Receives {type(inputs)} inputs.")
-
-        # Promote the inputs to a higher bitwidth to avoid saturation
-        inputs = inputs.promote(self.buffer_bitwidth)
+        # The only use case where GAP would receive a FixedPoint is when inputs are coming from an
+        # add layer and in that case they would necessarily be per-tensor.
+        if isinstance(inputs, FixedPoint):
+            inputs.assert_per_tensor()
         inputs_sum = tf.reduce_sum(inputs, axis=[1, 2], keepdims=self.keepdims)
-        inputs_mean = inputs_sum / self.spatial_size
-
-        if self.out_quantizer is not None:
-            return self.out_quantizer(inputs_mean)
-        return inputs_mean
+        return inputs_sum / self.spatial_size
 
     def get_config(self):
         config = super().get_config()
