@@ -20,7 +20,62 @@ import keras.backend as K
 from keras.layers import Layer
 
 from ..tensors import QTensor, QFloat, FixedPoint, ceil_log2
-from .layers import Calibrable, CalibrableVariable
+from .calibrable import Calibrable, CalibrableVariable
+
+
+__all__ = ["deserialize_quantizer", "Quantizer", "Dequantizer", "WeightQuantizer",
+           "OutputQuantizer"]
+
+
+def deserialize_quantizer(quant_config, name, quantizer_class, mandatory=True):
+    """Wrapper function of tf.keras.utils.deserialize_keras_object.
+
+    It allows to select the right config from the config file dict,
+    and raises an error if no config found or set to None.
+    If one is found, it returns the corresponding deserialized Quantizer.
+
+    Args:
+        quant_config (dict): quantization config dictionnary.
+        name (str): keras object name to deserialize.
+        quantizer_class (object): the Quantizer subclass to instantiate.
+        mandatory (bool, optional): flag to specify if the object to
+            deserialize is mandatory. If yes raises an Error otherwise
+            returns None. Defaults to True.
+
+    Returns:
+        :obj:`Quantizer`: the targeted deserialized Quantizer
+        object.
+    """
+    object_dict = quant_config.get(name)
+    if object_dict is None:
+        if mandatory:
+            raise KeyError(f"'{name}' should be specified.")
+        return None
+
+    # Check that object_dict has the right keys
+    list_available_keys = {
+        "trainable": bool,
+        "bitwidth": int,
+        "signed": bool,
+        "axis": str,
+        "momentum": float,
+        "dtype": str,
+        "scale_bits": int
+    }
+    for key in object_dict:
+        if key == "scale_bits" and quantizer_class == OutputQuantizer:
+            raise ValueError(f"'{name}' produces FixedPoint output. "
+                             "It doesn't support 'scale_bits' key.")
+        if key not in list_available_keys:
+            raise KeyError(f"'{key}' is not a valid key for '{name}'.")
+        if not isinstance(object_dict[key], list_available_keys[key]):
+            raise ValueError(f"'{key}' should be of type {list_available_keys[key]}. "
+                             f"Received {type(object_dict[key])}.")
+
+    # Deserialize the object
+    deserialize_dict = object_dict.copy()
+    deserialize_dict["name"] = name
+    return quantizer_class.from_config(deserialize_dict)
 
 
 @contextmanager
@@ -36,7 +91,7 @@ def disable_partitioner(layer):
             layer._scope.set_partitioner(partitioner)
 
 
-class Quantizer(Calibrable, Layer):
+class Quantizer(Layer):
     """The base class for all quantizers.
 
     The bitwidth defines the number of quantization levels on which the
@@ -44,15 +99,15 @@ class Quantizer(Calibrable, Layer):
     For a quantizer that accepts unsigned values, the maximum quantization
     level is 2 ^ bitwidth - 1.
     For a quantizer that accepts signed values, we lose one bit of precision to
-    store the sign, so the minimum/maximum level is -/+ 2 ^ (bitwidth - 1) - 1.
-    When the quantizer is signed, the quantization interval is always symmetric
-    around zero.
+    store the sign.
+    When the quantizer is signed, the quantization interval is asymmetric around
+    zero (i.e range: [- 2 ^ (bitwidth - 1), 2 ^ (bitwidth - 1) - 1]).
     The quantization is actually performed on absolute values, between 0 and
     max_value, where:
+
     - max_value is either a scalar (per-tensor quantization), or a vector
       (per-axis quantization),
-    - max_value is a static value on inference, and an adaptative value on training
-      (updated by the moving average algorithm, based on BatchNormalizationLayer).
+    - max_value is a static value on inference, and an adaptative value on training.
 
     Args:
         bitwidth (int): the quantization bitwidth.
@@ -60,24 +115,18 @@ class Quantizer(Calibrable, Layer):
             Defaults to True.
         axis (str, optional): reduce across all tensor values ('per-tensor') or keep the
             last axis ('per-axis'). Defaults to 'per-tensor'.
-        momentum (float, optional): the momentum for the moving average. Defaults to 0.9.
 
     Note:
         To get more information about the moving average implementation, see the
-        `BatchNormalizationLayer <https://bit.ly/3KcEaUh>`_ class:
+        `BatchNormalizationLayer <https://bit.ly/3KcEaUh>`_ class.
     """
 
-    # Follow BatchNormalizationLayer, the base class uses V2 behavior by default.
-    # To use V1 behavior, set to False.
-    _USE_V2_BEHAVIOR = True
-
-    def __init__(self, bitwidth, signed=True, axis="per-tensor", momentum=0.9, **kwargs):
+    def __init__(self, bitwidth, signed=True, axis="per-tensor", **kwargs):
         assert bitwidth > 1
         self.bitwidth = bitwidth
         self.signed = signed
         self.value_bits = bitwidth - 1 if signed else bitwidth
         self._axis = axis
-        self.momentum = momentum
         if not (isinstance(axis, str) and axis in ["per-tensor", "per-axis"]):
             raise ValueError(
                 f"Only support reduction 'per-tensor' or 'per-axis'. Given {axis}.")
@@ -95,6 +144,114 @@ class Quantizer(Calibrable, Layer):
         else:
             self.axis = None
 
+    def frac_bits(self, max_value):
+        # Clamp the max_value to the next power-of-two
+        int_bits = tf.cast(ceil_log2(max_value), tf.int32)
+        int_bits = tf.clip_by_value(int_bits, 0, self.value_bits)
+        # Evaluate fractional bits
+        return self.value_bits - int_bits
+
+    def get_config(self):
+        """Get the config of the layer.
+
+        Returns:
+            dict: the config of the layer.
+        """
+        config = super().get_config()
+        config.update({"bitwidth": self.bitwidth})
+        config.update({"signed": self.signed})
+        config.update({"axis": self._axis})
+        return config
+
+
+@tf.keras.utils.register_keras_serializable()
+class WeightQuantizer(Quantizer):
+    """A trivial uniform quantizer that has only one scale for everything.
+
+    Scale is dynamic (depends on inputs), and it can be read via the scale property.
+
+    Args:
+        bitwidth (int): the quantization bitwidth.
+        scale_bits (int, optional): the number of bits for the scaling. Defaults to None.
+    """
+
+    def __init__(self, bitwidth, scale_bits=None, **kwargs):
+        if not isinstance(bitwidth, int) or bitwidth < 2:
+            raise ValueError(
+                f"Bitwidth should be an int >= 2, currently {bitwidth}")
+        self.scale_bits = scale_bits
+        super().__init__(bitwidth, **kwargs)
+
+    def call(self, inputs, training=None):
+        """Update the max_value from the new inputs, and quantize them in three steps:
+
+            1. Check if the inputs are float values,
+            2. Compute the max_value,
+            3. Quantize the inputs.
+
+        Args:
+            inputs (:obj:`tensorflow.Tensor` or :obj:`QTensor`): the inputs tensor.
+            training (bool, optional): the training mode. Defaults to None.
+
+        Returns:
+            :obj:`tensorflow.Tensor`: the quantized tensor.
+        """
+        if isinstance(inputs, QTensor):
+            raise ValueError(
+                f"{type(inputs)} input is not supported. WeightQuantizer only accepts float"
+                " inputs.")
+
+        # Compute the max_value from the inputs
+        max_value = tf.math.reduce_max(tf.math.abs(inputs), self.axis)
+        # Return a QFloat if scale_bits provided
+        if self.scale_bits:
+            float_max = tf.stop_gradient(max_value)
+            return QFloat.quantize(inputs, float_max, self.value_bits, self.scale_bits)
+        # Return a FixedPoint if no scale_bits provided
+        frac_bits = tf.stop_gradient(self.frac_bits(max_value))
+        return FixedPoint.quantize(inputs, self.value_bits, frac_bits)
+
+    def get_config(self):
+        """Get the config of the layer.
+
+        Returns:
+            dict: the config of the layer.
+        """
+        config = super().get_config()
+        config.update({"scale_bits": self.scale_bits})
+        return config
+
+
+@tf.keras.utils.register_keras_serializable()
+class OutputQuantizer(Calibrable, Quantizer):
+    """A uniform quantizer that aligns its quantization range to a Power-of-two.
+    Its max_value is updated during the calibration by the moving average algorithm,
+    inspired by the BatchNormalizationLayer moving average algorithm.
+
+    Args:
+        bitwidth (int): the quantization bitwidth.
+        signed (bool): whether the quantizer expects signed values or unsigned.
+        momentum (float, optional): the momentum for the moving average. Defaults to 0.9.
+
+    Note:
+        To get more information about the moving average implementation, see the
+        `BatchNormalizationLayer <https://bit.ly/3KcEaUh>`_ class:
+    """
+
+    def __init__(self, momentum=0.9, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.momentum = momentum
+        # Add object that will store the shift values.
+        self.shift = CalibrableVariable()
+
+    def build(self, input_shape):
+        """Build the layer.
+
+        Args:
+            input_shape (list): the shape of input tensor.
+        """
+        super().build(input_shape)
+
         # Declares the constant/vector that will store the maximum values of the input.
         with disable_partitioner(self):
             self.max_value = self.add_weight(
@@ -108,6 +265,27 @@ class Quantizer(Calibrable, Layer):
                 experimental_autocast=False,
             )
 
+    @staticmethod
+    def _assign_new_value(variable, value):
+        """Given a variable, assign a new value to it. Function taken of
+        `BatchNormalizationLayer <https://bit.ly/3v0gzll>`_ code.
+
+        Args:
+            variable (:obj:`tensorflow.Variable`): the variable to assign.
+            value (:obj:`tensorflow.Tensor`): the new value to assign.
+
+        Returns:
+            :obj:`tensorflow.Tensor`: the new value of the variable.
+        """
+        with K.name_scope("AssignNewValue") as scope:
+            # Expected match shape
+            value_r = tf.reshape(value, tf.shape(variable))
+            if tf.compat.v1.executing_eagerly_outside_functions():
+                return variable.assign(value_r, name=scope)
+            else:
+                with tf.compat.v1.colocate_with(variable):
+                    return tf.compat.v1.assign(variable, value_r, name=scope)
+
     @staticmethod
     def _assign_moving_average(variable, value, momentum, inputs_size):
         """Given a variable, assign a new value to it, using a moving average.
@@ -142,66 +320,11 @@ class Quantizer(Calibrable, Layer):
                 with tf.compat.v1.colocate_with(variable):
                     return tf.compat.v1.assign_sub(variable, calculate_update_delta(), name=scope)
 
-    @staticmethod
-    def _assign_new_value(variable, value):
-        """Given a variable, assign a new value to it. Function taken of
-        `BatchNormalizationLayer <https://bit.ly/3v0gzll>`_ code.
-
-        Args:
-            variable (:obj:`tensorflow.Variable`): the variable to assign.
-            value (:obj:`tensorflow.Tensor`): the new value to assign.
-
-        Returns:
-            :obj:`tensorflow.Tensor`: the new value of the variable.
-        """
-        with K.name_scope("AssignNewValue") as scope:
-            # Expected match shape
-            value_r = tf.reshape(value, tf.shape(variable))
-            if tf.compat.v1.executing_eagerly_outside_functions():
-                return variable.assign(value_r, name=scope)
-            else:
-                with tf.compat.v1.colocate_with(variable):
-                    return tf.compat.v1.assign(variable, value_r, name=scope)
-
-    @staticmethod
-    def _update_weights(inputs, variable, axis, input_batch_size=None, momentum=0.9):
-        """Given the inputs tensor, a variable is updated with the maximum reduction of
-        the absolute input values. This reduction will be used on moving average algorithm.
-
-        Args:
-            inputs (:obj:`tensorflow.Tensor`): the inputs tensor.
-            variable (:obj:`tensorflow.Variable`): the variable to assign.
-            axis (List[int], int or None): the axis to make the reduction.
-            input_batch_size (int, optional): the batch size of the inputs. Defaults to None.
-            momentum (float, optional): the momentum for the moving average. Defaults to 0.9.
-
-        Note:
-            If ``momentum`` is None, the new value calculated over inputs will overwrite
-            the variable.
-        """
-        # Compute the new value for all weights
-        max_value = tf.math.reduce_max(tf.math.abs(inputs), axis)
-        if momentum == -1.:
-            Quantizer._assign_new_value(variable, max_value)
-        else:
-            Quantizer._assign_moving_average(
-                variable, max_value, momentum, input_batch_size)
-
-    def quantize(self, x):
-        """Quantize the input tensor.
-
-        Args:
-            x (:obj:`tensorflow.Tensor`): the input tensor.
-
-        Returns:
-            :obj:`tensorflow.Tensor`: the quantized tensor.
-        """
-        raise NotImplementedError
-
     def call(self, inputs, training=None):
-        """Update the weights from moving average, and quantize the inputs in three steps:
-            1. Convert the inputs to float values,
-            2. Update the weights from moving average, only if calibration is enabled and
+        """Update the max_value from moving average, and quantize the inputs in three steps:
+
+            1. Check that the inputs Tensor is a FixedPoint,
+            2. Update the max_value from moving average, only if calibration is enabled and
             3. Quantize the inputs.
 
         Args:
@@ -211,27 +334,40 @@ class Quantizer(Calibrable, Layer):
         Returns:
             :obj:`tensorflow.Tensor`: the quantized tensor.
         """
-        if isinstance(inputs, QTensor):
-            if inputs.value_bits <= self.value_bits:
-                msg = f"Quantizing a {inputs.value_bits}-bit QTensor to "\
-                      f"{self.value_bits}-bit is pointless."
-                if inputs.value_bits < self.value_bits:
-                    msg += " Use a promotion instead."
-                raise ValueError(msg)
+        if not isinstance(inputs, FixedPoint):
+            raise TypeError("The OutputQuantizer accepts only FixedPoint inputs."
+                            f"Received {type(inputs)} inputs.")
+
+        if inputs.value_bits <= self.value_bits:
+            msg = f"Quantizing a {inputs.value_bits}-bit QTensor to "\
+                f"{self.value_bits}-bit is pointless."
+            if inputs.value_bits < self.value_bits:
+                msg += " Use a promotion instead."
+            raise ValueError(msg)
+
         if self.calibration:
-            if isinstance(inputs, QTensor):
-                new_inputs = inputs.to_float()
-            else:
-                new_inputs = inputs
             # Retrieve information from the inputs and update the weights
-            input_batch_size = tf.shape(new_inputs)[0]
+            input_batch_size = tf.shape(inputs)[0]
             if tf.reduce_all(tf.math.equal(self.max_value, tf.constant(1.))):
                 momentum = tf.constant(-1.)
             else:
                 momentum = tf.convert_to_tensor(self.momentum)
-            self._update_weights(new_inputs, self.max_value,
-                                 self.axis, input_batch_size, momentum)
-        return self.quantize(inputs)
+            # Compute the new value for all weights
+            max_value = tf.math.reduce_max(tf.math.abs(inputs.to_float()), self.axis)
+            # If max_values never updated set their newly computed values otherwise
+            # update with moving average algorithm
+            if momentum == -1:
+                OutputQuantizer._assign_new_value(self.max_value, max_value)
+            else:
+                OutputQuantizer._assign_moving_average(
+                    self.max_value, max_value, momentum, input_batch_size)
+
+        # Quantize/downscale the input
+        frac_bits = tf.stop_gradient(self.frac_bits(self.max_value))
+        inputs, shift_value = inputs.downscale(frac_bits, self.value_bits)
+        # update shift values
+        self.shift(shift_value)
+        return inputs
 
     def get_config(self):
         """Get the config of the layer.
@@ -240,103 +376,14 @@ class Quantizer(Calibrable, Layer):
             dict: the config of the layer.
         """
         config = super().get_config()
-        config.update({"bitwidth": self.bitwidth})
-        config.update({"signed": self.signed})
-        config.update({"axis": self._axis})
         config.update({"momentum": self.momentum})
         return config
 
 
-@tf.keras.utils.register_keras_serializable()
-class WeightQuantizer(Quantizer):
-    """A trivial uniform quantizer that has only one scale for everything.
-    Scale is dynamic (depends on inputs), and it can be read via the scale property.
-
-    Args:
-        bitwidth (int): the quantization bitwidth.
-        scale_bits (int, optional): the number of bits for the scaling. Defaults to 24.
-    """
-
-    def __init__(self, bitwidth, scale_bits=24, **kwargs):
-        if not isinstance(bitwidth, int) or bitwidth < 2:
-            raise ValueError(
-                f"Bitwidth should be an int >= 2, currently {bitwidth}")
-        self.scale_bits = scale_bits
-        super().__init__(bitwidth, **kwargs)
-
-    def quantize(self, x):
-        """Quantize float inputs
-
-        Args:
-            x(:obj:`tensorflow.Tensor`): a Tensor of float inputs.
-
-        Returns:
-            a QFloat
-        """
-        if isinstance(x, QTensor):
-            raise ValueError(f"{type(x)} input is not supported. WeightQuantizer only accepts float"
-                             " inputs.")
-        float_max = tf.stop_gradient(self.max_value)
-        return QFloat.quantize(x, float_max, self.value_bits, self.scale_bits)
-
-    def get_config(self):
-        """Get the config of the layer.
-
-        Returns:
-            dict: the config of the layer.
-        """
-        config = super().get_config()
-        config.update({"scale_bits": self.scale_bits})
-        return config
-
-
-@tf.keras.utils.register_keras_serializable()
-class FixedPointQuantizer(Quantizer):
-    """A uniform quantizer that aligns its quantization range to a Power-of-two
-
-    Args:
-        bitwidth (int): the quantization bitwidth.
-        signed (bool): whether the quantizer expects signed values or unsigned.
-    """
-
-    def __init__(self,  *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        # Add object that will store the shift values.
-        self.shift = CalibrableVariable()
-
-    @property
-    def int_bits(self):
-        # Clamp the max_value to the next power-of-two
-        int_bits = tf.cast(ceil_log2(self.max_value), tf.int32)
-        return tf.clip_by_value(int_bits, 0, self.value_bits)
-
-    @property
-    def frac_bits(self):
-        # Evaluate fractional bits
-        return self.value_bits - self.int_bits
-
-    def quantize(self, x):
-        """Quantize float inputs
-
-        Args:
-            x(:obj:`tensorflow.Tensor`): a Tensor of float inputs.
-
-        Returns:
-            a FixedPoint
-        """
-        if isinstance(x, QFloat):
-            x = x.to_float()
-        frac_bits = tf.stop_gradient(self.frac_bits)
-        if isinstance(x, FixedPoint):
-            x, shift_value = x.downscale(frac_bits, self.value_bits)
-            # update shift values
-            self.shift(shift_value)
-            return x
-        return FixedPoint.quantize(x, frac_bits, self.value_bits)
-
-
 @tf.keras.utils.register_keras_serializable()
 class Dequantizer(Layer):
+    """ Layer that allows to dequantize its inputs.
+    """
 
     def call(self, inputs, training=None):
         """Convert QTensor inputs to float.
@@ -346,7 +393,7 @@ class Dequantizer(Layer):
             training (bool, optional): the training mode. Defaults to None.
 
         Returns:
-            :obj:`tensorflow.Tensor`: the quantized tensor(s).
+            :obj:`tensorflow.Tensor`: the dequantized tensor(s).
         """
         def dequantize(x):
             if isinstance(x, QTensor):
