@@ -14,6 +14,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ******************************************************************************
+
+__all__ = ["AlignedWeightQuantizer"]
+
 import tensorflow as tf
 
 from ...tensors import QFloat
@@ -21,9 +24,6 @@ from ..recorders import TensorRecorder, QFloatRecorder
 from .quantizers import Quantizer
 
 
-__all__ = ["AlignedWeightQuantizer"]
-
-
 @tf.keras.utils.register_keras_serializable()
 class AlignedWeightQuantizer(Quantizer):
     """A uniform quantizer that converts a float Tensor to a QTensor representation.
@@ -33,12 +33,12 @@ class AlignedWeightQuantizer(Quantizer):
     QFloat input.
 
     Args:
-        bitwidth (int): the quantization bitwidth.
+        bitwidth (int, optional): the quantization bitwidth. Defaults to 8.
         signed (bool, optional): whether the quantizer expects signed values or unsigned.
             Defaults to True.
     """
 
-    def __init__(self, bitwidth, signed=True, **kwargs):
+    def __init__(self, bitwidth=8, signed=True, **kwargs):
         super().__init__(bitwidth, signed, **kwargs)
         # Add the object that will store the shift values.
         self.shift = TensorRecorder()
