@@ -62,9 +62,9 @@ class OutputQuantizer(Calibrable, Quantizer):
     def __init__(self, bitwidth=8, signed=True, axis="per-tensor",
                  momentum=0.9, scale_bits=8, **kwargs):
         super().__init__(bitwidth, signed, **kwargs)
-        self._axis = axis
         if not (isinstance(axis, str) and axis in ["per-tensor", "per-axis"]):
             raise ValueError(f"Only support reduction 'per-tensor' or 'per-axis'. Given {axis}.")
+        self.axis = axis
         self.momentum = momentum
         self.scale_bits = scale_bits
         # Add object that will store the shift values.
@@ -77,16 +77,20 @@ class OutputQuantizer(Calibrable, Quantizer):
             input_shape (list): the shape of input tensor.
         """
         # Convert axis to a list of int
-        if self._axis == "per-axis" and len(input_shape) > 1:
-            self.axis = list(range(len(input_shape) - 1))
+        if self.axis == "per-axis":
+            ndims = len(input_shape)
+            if ndims < 3:
+                raise ValueError("OutputQuantizer cannot quantize per-axis tensors "
+                                 " with 2 dimensions or less.")
+            self._axis = list(range(len(input_shape) - 1))
         else:
-            self.axis = None
+            self._axis = None
 
         # Declares the constant/vector that will store the maximum values of the input.
         with disable_partitioner(self):
             self.max_value = self.add_weight(
                 name="max_value",
-                shape=input_shape[-1] if self.axis is not None else (),
+                shape=input_shape[-1] if self._axis is not None else (),
                 dtype=tf.float32,
                 initializer="ones",
                 synchronization=tf.VariableSynchronization.ON_READ,
@@ -196,7 +200,7 @@ class OutputQuantizer(Calibrable, Quantizer):
             else:
                 momentum = tf.convert_to_tensor(self.momentum)
             # Compute the new value for all weights
-            max_value = tf.math.reduce_max(tf.math.abs(inputs.to_float()), self.axis)
+            max_value = tf.math.reduce_max(tf.math.abs(inputs.to_float()), self._axis)
             # If max_values never updated set their newly computed values otherwise
             # update with moving average algorithm
             if momentum == -1:
@@ -221,5 +225,5 @@ class OutputQuantizer(Calibrable, Quantizer):
         config = super().get_config()
         config.update({"momentum": self.momentum})
         config.update({"scale_bits": self.scale_bits})
-        config.update({"axis": self._axis})
+        config.update({"axis": self.axis})
         return config
