@@ -18,7 +18,7 @@ import keras
 import tensorflow as tf
 
 from ..tensors import ceil_log2, FixedPoint, MAX_BUFFER_BITWIDTH
-from .calibrable import Calibrable, CalibrableVariable
+from .recorders import TensorRecorder
 from .quantizers import deserialize_quantizer, OutputQuantizer
 from ..debugging import assert_less_equal
 
@@ -31,12 +31,12 @@ class Reciprocal(keras.layers.Layer):
     """Layer that computes the reciprocal of the input.
     """
 
-    def call(self, inputs, training=False):
+    def call(self, inputs):
         return tf.math.reciprocal(inputs)
 
 
 @tf.keras.utils.register_keras_serializable()
-class QuantizedReciprocal(Calibrable, Reciprocal):
+class QuantizedReciprocal(Reciprocal):
     """Piece-wise approximation of y = 1/x.
 
     The approximation works on values in the range [1, 2).
@@ -91,7 +91,7 @@ class QuantizedReciprocal(Calibrable, Reciprocal):
         self._limit_1_5 = FixedPoint.quantize(1.5, value_bits, frac_bits - mul_frac_bits)
 
         # We need to store the input frac bits to reproduce the reciprocal on hardware.
-        self.input_frac_bits = CalibrableVariable()
+        self.input_frac_bits = TensorRecorder()
 
     def _reciprocal_x_ge_1_5_values(self, x):
         """Implements reciprocal approximation when x in [1.5, 2.0), following the equation:
@@ -123,7 +123,7 @@ class QuantizedReciprocal(Calibrable, Reciprocal):
         x_reciprocal = x + self._K_1_59375
         return x_reciprocal.values
 
-    def call(self, inputs, training=False):
+    def call(self, inputs):
         if not isinstance(inputs, FixedPoint):
             raise TypeError("QuantizedReciprocal only accepts FixedPoint inputs. "
                             f"Receives '{type(inputs)}' as inputs.")
@@ -173,7 +173,7 @@ class QuantizedReciprocal(Calibrable, Reciprocal):
         #
         y_values = inputs.shift(x_aligned_bits - used_bits).values
         y_frac_bits = x_aligned_bits - 1
-        y = FixedPoint(y_values, y_frac_bits, x_value_bits)
+        y = FixedPoint(y_values, x_value_bits, y_frac_bits)
 
         # Estimate output values using one of the PWL functions
         reciprocal_y_values = tf.where(y >= self._limit_1_5,
@@ -187,7 +187,7 @@ class QuantizedReciprocal(Calibrable, Reciprocal):
         #              = x_value_bits - 1
         # This is sure given the fact that the range of reciprocal_y is within (0,1)
         y_frac_bits = x_value_bits - 1
-        reciprocal_y = FixedPoint(reciprocal_y_values, y_frac_bits, x_value_bits)
+        reciprocal_y = FixedPoint(reciprocal_y_values, x_value_bits, y_frac_bits)
 
         # Since we defined previously
         #
