@@ -14,16 +14,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ******************************************************************************
+
+__all__ = ["UnfoldPatches", "QuantizedUnfoldPatches", "FoldPatches", "QuantizedFoldPatches"]
+
 import keras
 import tensorflow as tf
 
+from ..decorators import (register_quantize_target, tensor_inputs, register_no_output_quantizer,
+                          register_aligned_inputs)
 from ...tensors import FixedPoint
 from ...debugging import assert_equal
 
 
-__all__ = ["UnfoldPatches", "QuantizedUnfoldPatches", "FoldPatches", "QuantizedFoldPatches"]
-
-
 @tf.keras.utils.register_keras_serializable()
 class UnfoldPatches(keras.layers.Layer):
     """ Unfold an input feature map into patches.
@@ -69,16 +71,16 @@ class UnfoldPatches(keras.layers.Layer):
         return config
 
 
+@register_quantize_target(UnfoldPatches)
+@register_no_output_quantizer
+@register_aligned_inputs
 @tf.keras.utils.register_keras_serializable()
 class QuantizedUnfoldPatches(UnfoldPatches):
     """An UnfoldPatches layer that operates on quantized inputs
 
     """
+    @tensor_inputs([FixedPoint])
     def call(self, inputs):
-        # raise an error if the inputs are not FixedPoint
-        if not isinstance(inputs, FixedPoint):
-            raise TypeError(f"QuantizedUnfoldPatches only accepts FixedPoint inputs.\
-                            Receives {type(inputs)} inputs.")
         inputs_shape = tf.shape(inputs)
         batch_size = inputs_shape[0]
         channels = inputs_shape[3]
@@ -141,6 +143,9 @@ class FoldPatches(keras.layers.Layer):
         return config
 
 
+@register_quantize_target(FoldPatches)
+@register_no_output_quantizer
+@register_aligned_inputs
 @tf.keras.utils.register_keras_serializable()
 class QuantizedFoldPatches(FoldPatches):
     """A FoldPatches layer that operates on quantized inputs
