@@ -20,9 +20,9 @@ import tensorflow as tf
 
 from keras import Model, layers
 
-from .utils import deep_clone_model, insert_layer, find_layer_config
-from .transforms import (align_rescaling, invert_batchnorm_pooling, fold_batchnorms,
-                         align_relu_max_value, remove_zeropadding2d)
+from .utils import deep_clone_model, find_layer_config
+from .transforms import (insert_layer, align_rescaling, invert_batchnorm_pooling, fold_batchnorms,
+                         align_relu_max_value, remove_zeropadding2d, invert_relu_maxpool)
 from ..layers import (OutputQuantizer, WeightQuantizer, QuantizedReLU, Dequantizer, quantization,
                       QuantizationParams, get_quantization_params, UnfoldPatches, FoldPatches,
                       Attention, QuantizedConv2D, QuantizedDepthwiseConv2D,
@@ -136,17 +136,23 @@ def _prepare_output_quantizers(model):
     def set_output_quantizer(layer_names, next_layer):
         """ Populates `out_quantizer_configs` with layer names and their OutputQuantizer. """
         for name in layer_names:
-            # Handle special cases were the OutputQuantizer must be per-tensor, which is when
-            # next_layer is:
-            #   - an UnfoldPatches or FoldPatches layer
-            #   - an Attention layer and the layer is Query or Key (first and second inputs)
-            patches_next = isinstance(next_layer, (UnfoldPatches, FoldPatches))
             current_layer = model.get_layer(name)
+            # Handle special cases were the OutputQuantizer must be per-tensor:
+            # - when current_layer has vector outputs,
+            # - when next_layer is an UnfoldPatches or FoldPatches layer
+            # - when next_layer is an Attention layer and the layer is Query or Key
+            #   (first and second inputs)
+            if isinstance(current_layer, Attention):
+                output_shape = current_layer.output_shape[0]
+            else:
+                output_shape = current_layer.output_shape
+            vector_outputs = len(output_shape) < 3
+            patches_next = isinstance(next_layer, (UnfoldPatches, FoldPatches))
             query_or_key = (isinstance(current_layer, layers.Dense)
                             and isinstance(next_layer, Attention)
                             and next_layer.inbound_nodes[0].inbound_layers.
                             index(current_layer) in [0, 1])
-            per_tensor = patches_next or query_or_key
+            per_tensor = patches_next or query_or_key or vector_outputs
 
             if isinstance(current_layer, layers.ReLU):
                 per_tensor = per_tensor or qparams.per_tensor_activations
@@ -193,18 +199,10 @@ def _prepare_output_quantizers(model):
                 previous_layers.append(inbound.name)
         return previous_layers
 
-    # Handle the model output: last layer in a model must have an OutputQuantizer
-    last_layer = model.layers[-1]
-    if cannot_have_output_quantizer(last_layer):
-        last_layer_names = get_preceding_layer_names(last_layer)
-    else:
-        last_layer_names = [last_layer.name]
-    set_output_quantizer(last_layer_names, last_layer)
-
     # Parse the layers in reverse order
     for layer in model.layers[::-1]:
         # Find layers that will need aligned inputs
-        if isinstance(layer, Dequantizer) or get_quantized_layer(layer) in _GLOBAL_ALIGNED_INPUTS:
+        if get_quantized_layer(layer) in _GLOBAL_ALIGNED_INPUTS:
             # Retrieve the inbounds that can have an OutputQuantizer
             previous_layers = get_preceding_layer_names(layer)
             # Set an OutputQuantizer in their inbounds
@@ -213,7 +211,7 @@ def _prepare_output_quantizers(model):
     return out_quantizer_configs
 
 
-def quantize(model, q_config=None, qparams=QuantizationParams(), add_dequantizer=True):
+def quantize(model, q_config=None, qparams=QuantizationParams()):
     """Quantizes the model using the provided configuration or parameters.
 
     Details on how this function behaves:
@@ -231,12 +229,13 @@ def quantize(model, q_config=None, qparams=QuantizationParams(), add_dequantizer
     While the name matching will first look for exact name matches, layer names keys can also be
     expressed as regular expressions to match with several layers.
 
+    If not already present, a final Dequantizer will be added at the end of the Model.
+
     Args:
         model (keras.Model): the model to quantize
         q_config (dict, optional): quantization configuration. Defaults to None.
         qparams (QuantizationParams, optional): global quantization parameters. Defaults to
             QuantizationParams().
-        add_dequantizer (bool, optional): allows to convert output to float. Defaults to True.
 
     Returns:
         keras.Model: the quantized model
@@ -328,6 +327,9 @@ def quantize(model, q_config=None, qparams=QuantizationParams(), add_dequantizer
     # Align Rescaling (if needed)
     model = align_rescaling(model)
 
+    # Invert ReLU <-> MaxPool layers so that MaxPool comes first
+    model = invert_relu_maxpool(model)
+
     # Invert BN <-> Pooling layers and fold BN into their preceding layers
     model = invert_batchnorm_pooling(model)
     model = fold_batchnorms(model)
@@ -349,8 +351,7 @@ def quantize(model, q_config=None, qparams=QuantizationParams(), add_dequantizer
         out = new_model.outputs
 
         # Append Dequantizer at the end of the model to convert the output to float value
-        if (add_dequantizer and not isinstance(new_model.layers[-1], Dequantizer)
-                and not quantization_stopped):
+        if not isinstance(new_model.layers[-1], Dequantizer) and not quantization_stopped:
             out = Dequantizer()(out)
 
         # Build the model
