@@ -2,3 +2,5 @@ from .rescaling import *
 from .batch_normalization import *
 from .align_relu import *
 from .remove_zeropadding2d import *
+from .relu_maxpool import *
+from .insert_layer import *
