@@ -25,8 +25,7 @@ from copy import deepcopy
 from keras.models import Sequential
 from keras.layers import ZeroPadding2D, Conv2D, SeparableConv2D, DepthwiseConv2D
 
-from ..utils import (deep_clone_model, _get_layers, _get_layer_index, _inbound_node_generator,
-                     _replace_layer_name_for_connection_info, get_layers_by_type)
+from .transforms_utils import get_layers, get_layer_index, get_layers_by_type, update_inbound
 
 
 def _find_removable_zeropadding(model):
@@ -95,7 +94,7 @@ def _get_zeropadding_less_model(model, map_zeropadding_next):
 
     for zeropadding, next_layer in map_zeropadding_next.items():
         # Set padding='same' in the layer that follows a ZeroPadding that will be removed
-        next_index = _get_layer_index(layers, next_layer.name)
+        next_index = get_layer_index(layers, next_layer.name)
         layers[next_index]['config']['padding'] = 'same'
 
         # For sequential model, the changes stop here: the ZeroPadding2D layers will simply be
@@ -105,7 +104,7 @@ def _get_zeropadding_less_model(model, map_zeropadding_next):
             continue
 
         # Retrieve the ZeroPadding2D input layer, assuming it has only 1 inbound
-        zeropadding_index = _get_layer_index(layers, zeropadding.name)
+        zeropadding_index = get_layer_index(layers, zeropadding.name)
         # tfmot code: 'inbound_nodes' is a nested list where first element is the inbound layername,
         # e.g: [[['conv1', 0, 0, {} ]]]
         updated_inbound = layers[zeropadding_index]['inbound_nodes'][0][0][0]
@@ -114,15 +113,10 @@ def _get_zeropadding_less_model(model, map_zeropadding_next):
         # that will be removed so it must be replaced with the ZeroPadding2D previous layer. This
         # results in by-passing the ZeroPadding2D layer: inbound > ZeroPadding2D > outbounds becomes
         # inbound > outbounds.
-        for inbound_node in _inbound_node_generator(layers[next_index]):
-            if isinstance(inbound_node, dict):
-                inbound_node = inbound_node.values()
-            for connection_info in inbound_node:
-                _replace_layer_name_for_connection_info(connection_info, zeropadding.name,
-                                                        updated_inbound)
+        update_inbound(layers[next_index], zeropadding.name, updated_inbound)
 
     # Remove ZeroPadding2D layers
-    layers_to_remove = _get_layers(config, [zp.name for zp in map_zeropadding_next.keys()])
+    layers_to_remove = get_layers(config, [zp.name for zp in map_zeropadding_next.keys()])
     for layer_to_remove in layers_to_remove:
         layers.remove(layer_to_remove)
 
@@ -155,11 +149,8 @@ def remove_zeropadding2d(model):
     if not map_zeropadding_next:
         return model
 
-    # Clone the model to apply modifications
-    clone_model = deep_clone_model(model)
-
     # Rebuild a model without ZeroPadding2D by editing the configuration
-    updated_model = _get_zeropadding_less_model(clone_model, map_zeropadding_next)
+    updated_model = _get_zeropadding_less_model(model, map_zeropadding_next)
 
     # Restore model weights
     updated_model.set_weights(model.get_weights())
