@@ -16,7 +16,7 @@
 # ******************************************************************************
 import tensorflow as tf
 
-from .qtensor import QTensor, round_through, floor_through, ceil_through, pow2, ceil_log2
+from .qtensor import QTensor, round_through, floor_through, pow2, ceil_log2
 from ..debugging import assert_equal, assert_none_equal, assert_less, assert_less_equal
 
 
@@ -92,7 +92,7 @@ class FixedPoint(QTensor):
         return frac_bits
 
     @staticmethod
-    def quantize(x, value_bits, frac_bits=None, ceil=False):
+    def quantize(x, value_bits, frac_bits=None):
         r"""Converts a float Tensor to a FixedPoint
 
         It converts the original float values into integer values so that:
@@ -124,7 +124,6 @@ class FixedPoint(QTensor):
             value_bits (int): the number of value bits
             frac_bits (:obj:`tensorflow.Tensor`, optional): an integer tensor of fractional bits.
                 Defaults to None.
-            ceil (bool, optional): ceil values instead of rounding them. Defaults to False.
 
         Returns:
             a :obj:`FixedPoint`
@@ -138,7 +137,7 @@ class FixedPoint(QTensor):
         # Project float into fixed-point representation space
         x_scaled = x * pow2(frac_bits)
         # Round or ceil to obtain integer values
-        values = ceil_through(x_scaled) if ceil else round_through(x_scaled)
+        values = round_through(x_scaled)
         return FixedPoint(values, value_bits, frac_bits)
 
     @property
