@@ -76,6 +76,6 @@ def reciprocal_lut(
     # float(1/x) ~= reciprocal_lut(x.values) * 2^x.frac_bits
     # in two steps:
     # 1. Compute the reciprocal of x.values of absolute value
-    x_inv = FixedPoint(hash_table(x.abs().values), frac_bits, out_value_bits)
+    x_inv = FixedPoint(hash_table(x.abs().values), out_value_bits, frac_bits)
     # 2. Scale by 2**x.frac_bits
-    return x_inv * FixedPoint(pow2(x.frac_bits)) * x.sign
+    return x_inv * FixedPoint(pow2(x.frac_bits), 32, 0) * x.sign
