@@ -374,8 +374,8 @@ def fp_depthwise_conv2d(x: FixedPoint,
         :obj:`FixedPoint`: output tensor.
 
     """
-    # x must be quantized per-tensor because the products are eventually summed
-    x.assert_per_tensor()
+    # Unlike other convolutions, the depthwise does not require its inputs to
+    # be quantized per-tensor as the input channels are processed independently
 
     if isinstance(depthwise_kernel, FixedPoint):
         filters_frac_bits = depthwise_kernel.frac_bits
